@@ -8,7 +8,7 @@ from js_example import app
 @app.route("/", defaults={"js": "plain"})
 @app.route("/<any(plain, jquery, fetch):js>")
 def index(js):
-    return render_template("{0}.html".format(js), js=js)
+    return render_template(f"{js}.html", js=js)
 
 
 @app.route("/add", methods=["POST"])
