@@ -1,31 +1,27 @@
-# -*- coding: utf-8 -*-
-"""
-    flask.ctx
-    ~~~~~~~~~
-
-    Implements the objects required to keep the context.
-
-    :copyright: 2010 Pallets
-    :license: BSD-3-Clause
-"""
 import sys
+import typing as t
 from functools import update_wrapper
+from types import TracebackType
 
 from werkzeug.exceptions import HTTPException
 
-from ._compat import BROKEN_PYPY_CTXMGR_EXIT
-from ._compat import reraise
 from .globals import _app_ctx_stack
 from .globals import _request_ctx_stack
 from .signals import appcontext_popped
 from .signals import appcontext_pushed
+from .typing import AfterRequestCallable
+
+if t.TYPE_CHECKING:
+    from .app import Flask
+    from .sessions import SessionMixin
+    from .wrappers import Request
 
 
 # a singleton sentinel value for parameter defaults
 _sentinel = object()
 
 
-class _AppCtxGlobals(object):
+class _AppCtxGlobals:
     """A plain object. Used as a namespace for storing data during an
     application context.
 
@@ -45,7 +41,25 @@ class _AppCtxGlobals(object):
         .. versionadded:: 0.10
     """
 
-    def get(self, name, default=None):
+    # Define attr methods to let mypy know this is a namespace object
+    # that has arbitrary attributes.
+
+    def __getattr__(self, name: str) -> t.Any:
+        try:
+            return self.__dict__[name]
+        except KeyError:
+            raise AttributeError(name) from None
+
+    def __setattr__(self, name: str, value: t.Any) -> None:
+        self.__dict__[name] = value
+
+    def __delattr__(self, name: str) -> None:
+        try:
+            del self.__dict__[name]
+        except KeyError:
+            raise AttributeError(name) from None
+
+    def get(self, name: str, default: t.Optional[t.Any] = None) -> t.Any:
         """Get an attribute by name, or a default value. Like
         :meth:`dict.get`.
 
@@ -56,12 +70,12 @@ class _AppCtxGlobals(object):
         """
         return self.__dict__.get(name, default)
 
-    def pop(self, name, default=_sentinel):
+    def pop(self, name: str, default: t.Any = _sentinel) -> t.Any:
         """Get and remove an attribute by name. Like :meth:`dict.pop`.
 
         :param name: Name of attribute to pop.
         :param default: Value to return if the attribute is not present,
-            instead of raise a ``KeyError``.
+            instead of raising a ``KeyError``.
 
         .. versionadded:: 0.11
         """
@@ -70,32 +84,32 @@ class _AppCtxGlobals(object):
         else:
             return self.__dict__.pop(name, default)
 
-    def setdefault(self, name, default=None):
+    def setdefault(self, name: str, default: t.Any = None) -> t.Any:
         """Get the value of an attribute if it is present, otherwise
         set and return a default value. Like :meth:`dict.setdefault`.
 
         :param name: Name of attribute to get.
-        :param: default: Value to set and return if the attribute is not
+        :param default: Value to set and return if the attribute is not
             present.
 
         .. versionadded:: 0.11
         """
         return self.__dict__.setdefault(name, default)
 
-    def __contains__(self, item):
+    def __contains__(self, item: str) -> bool:
         return item in self.__dict__
 
-    def __iter__(self):
+    def __iter__(self) -> t.Iterator[str]:
         return iter(self.__dict__)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         top = _app_ctx_stack.top
         if top is not None:
-            return "<flask.g of %r>" % top.app.name
+            return f"<flask.g of {top.app.name!r}>"
         return object.__repr__(self)
 
 
-def after_this_request(f):
+def after_this_request(f: AfterRequestCallable) -> AfterRequestCallable:
     """Executes a function after this request.  This is useful to modify
     response objects.  The function is passed the response object and has
     to return the same or a new one.
@@ -120,7 +134,7 @@ def after_this_request(f):
     return f
 
 
-def copy_current_request_context(f):
+def copy_current_request_context(f: t.Callable) -> t.Callable:
     """A helper function that decorates a function to retain the current
     request context.  This is useful when working with greenlets.  The moment
     the function is decorated a copy of the request context is created and
@@ -160,7 +174,7 @@ def copy_current_request_context(f):
     return update_wrapper(wrapper, f)
 
 
-def has_request_context():
+def has_request_context() -> bool:
     """If you have code that wants to test if a request context is there or
     not this function can be used.  For instance, you may want to take advantage
     of request information if the request object is available, but fail
@@ -192,7 +206,7 @@ def has_request_context():
     return _request_ctx_stack.top is not None
 
 
-def has_app_context():
+def has_app_context() -> bool:
     """Works like :func:`has_request_context` but for the application
     context.  You can also just do a boolean check on the
     :data:`current_app` object instead.
@@ -202,7 +216,7 @@ def has_app_context():
     return _app_ctx_stack.top is not None
 
 
-class AppContext(object):
+class AppContext:
     """The application context binds an application object implicitly
     to the current thread or greenlet, similar to how the
     :class:`RequestContext` binds request information.  The application
@@ -211,7 +225,7 @@ class AppContext(object):
     context.
     """
 
-    def __init__(self, app):
+    def __init__(self, app: "Flask") -> None:
         self.app = app
         self.url_adapter = app.create_url_adapter(None)
         self.g = app.app_ctx_globals_class()
@@ -220,15 +234,13 @@ class AppContext(object):
         # but there a basic "refcount" is enough to track them.
         self._refcnt = 0
 
-    def push(self):
+    def push(self) -> None:
         """Binds the app context to the current context."""
         self._refcnt += 1
-        if hasattr(sys, "exc_clear"):
-            sys.exc_clear()
         _app_ctx_stack.push(self)
         appcontext_pushed.send(self.app)
 
-    def pop(self, exc=_sentinel):
+    def pop(self, exc: t.Optional[BaseException] = _sentinel) -> None:  # type: ignore
         """Pops the app context."""
         try:
             self._refcnt -= 1
@@ -238,21 +250,20 @@ class AppContext(object):
                 self.app.do_teardown_appcontext(exc)
         finally:
             rv = _app_ctx_stack.pop()
-        assert rv is self, "Popped wrong app context.  (%r instead of %r)" % (rv, self)
+        assert rv is self, f"Popped wrong app context.  ({rv!r} instead of {self!r})"
         appcontext_popped.send(self.app)
 
-    def __enter__(self):
+    def __enter__(self) -> "AppContext":
         self.push()
         return self
 
-    def __exit__(self, exc_type, exc_value, tb):
+    def __exit__(
+        self, exc_type: type, exc_value: BaseException, tb: TracebackType
+    ) -> None:
         self.pop(exc_value)
 
-        if BROKEN_PYPY_CTXMGR_EXIT and exc_type is not None:
-            reraise(exc_type, exc_value, tb)
 
-
-class RequestContext(object):
+class RequestContext:
     """The request context contains all request relevant information.  It is
     created at the beginning of the request and pushed to the
     `_request_ctx_stack` and removed at the end of it.  It will create the
@@ -282,7 +293,13 @@ class RequestContext(object):
     that situation, otherwise your unittests will leak memory.
     """
 
-    def __init__(self, app, environ, request=None, session=None):
+    def __init__(
+        self,
+        app: "Flask",
+        environ: dict,
+        request: t.Optional["Request"] = None,
+        session: t.Optional["SessionMixin"] = None,
+    ) -> None:
         self.app = app
         if request is None:
             request = app.request_class(environ)
@@ -299,7 +316,7 @@ class RequestContext(object):
         # other request contexts.  Now only if the last level is popped we
         # get rid of them.  Additionally if an application context is missing
         # one is created implicitly so for each level we add this information
-        self._implicit_app_ctx_stack = []
+        self._implicit_app_ctx_stack: t.List[t.Optional["AppContext"]] = []
 
         # indicator if the context was preserved.  Next time another context
         # is pushed the preserved context is popped.
@@ -312,17 +329,17 @@ class RequestContext(object):
         # Functions that should be executed after the request on the response
         # object.  These will be called before the regular "after_request"
         # functions.
-        self._after_request_functions = []
+        self._after_request_functions: t.List[AfterRequestCallable] = []
 
     @property
-    def g(self):
+    def g(self) -> AppContext:
         return _app_ctx_stack.top.g
 
     @g.setter
-    def g(self, value):
+    def g(self, value: AppContext) -> None:
         _app_ctx_stack.top.g = value
 
-    def copy(self):
+    def copy(self) -> "RequestContext":
         """Creates a copy of this request context with the same request object.
         This can be used to move a request context to a different greenlet.
         Because the actual request object is the same this cannot be used to
@@ -342,17 +359,17 @@ class RequestContext(object):
             session=self.session,
         )
 
-    def match_request(self):
+    def match_request(self) -> None:
         """Can be overridden by a subclass to hook into the matching
         of the request.
         """
         try:
-            result = self.url_adapter.match(return_rule=True)
-            self.request.url_rule, self.request.view_args = result
+            result = self.url_adapter.match(return_rule=True)  # type: ignore
+            self.request.url_rule, self.request.view_args = result  # type: ignore
         except HTTPException as e:
             self.request.routing_exception = e
 
-    def push(self):
+    def push(self) -> None:
         """Binds the request context to the current context."""
         # If an exception occurs in debug mode or if context preservation is
         # activated under exception situations exactly one context stays
@@ -376,9 +393,6 @@ class RequestContext(object):
         else:
             self._implicit_app_ctx_stack.append(None)
 
-        if hasattr(sys, "exc_clear"):
-            sys.exc_clear()
-
         _request_ctx_stack.push(self)
 
         # Open the session at the moment that the request context is available.
@@ -392,10 +406,12 @@ class RequestContext(object):
             if self.session is None:
                 self.session = session_interface.make_null_session(self.app)
 
+        # Match the request URL after loading the session, so that the
+        # session is available in custom URL converters.
         if self.url_adapter is not None:
             self.match_request()
 
-    def pop(self, exc=_sentinel):
+    def pop(self, exc: t.Optional[BaseException] = _sentinel) -> None:  # type: ignore
         """Pops the request context and unbinds it by doing that.  This will
         also trigger the execution of functions registered by the
         :meth:`~flask.Flask.teardown_request` decorator.
@@ -404,9 +420,9 @@ class RequestContext(object):
            Added the `exc` argument.
         """
         app_ctx = self._implicit_app_ctx_stack.pop()
+        clear_request = False
 
         try:
-            clear_request = False
             if not self._implicit_app_ctx_stack:
                 self.preserved = False
                 self._preserved_exc = None
@@ -414,13 +430,6 @@ class RequestContext(object):
                     exc = sys.exc_info()[1]
                 self.app.do_teardown_request(exc)
 
-                # If this interpreter supports clearing the exception information
-                # we do that now.  This will only go into effect on Python 2.x,
-                # on 3.x it disappears automatically at the end of the exception
-                # stack.
-                if hasattr(sys, "exc_clear"):
-                    sys.exc_clear()
-
                 request_close = getattr(self.request, "close", None)
                 if request_close is not None:
                     request_close()
@@ -437,25 +446,26 @@ class RequestContext(object):
             if app_ctx is not None:
                 app_ctx.pop(exc)
 
-            assert rv is self, "Popped wrong request context. (%r instead of %r)" % (
-                rv,
-                self,
-            )
+            assert (
+                rv is self
+            ), f"Popped wrong request context. ({rv!r} instead of {self!r})"
 
-    def auto_pop(self, exc):
+    def auto_pop(self, exc: t.Optional[BaseException]) -> None:
         if self.request.environ.get("flask._preserve_context") or (
             exc is not None and self.app.preserve_context_on_exception
         ):
             self.preserved = True
-            self._preserved_exc = exc
+            self._preserved_exc = exc  # type: ignore
         else:
             self.pop(exc)
 
-    def __enter__(self):
+    def __enter__(self) -> "RequestContext":
         self.push()
         return self
 
-    def __exit__(self, exc_type, exc_value, tb):
+    def __exit__(
+        self, exc_type: type, exc_value: BaseException, tb: TracebackType
+    ) -> None:
         # do not pop the request stack if we are in debug mode and an
         # exception happened.  This will allow the debugger to still
         # access the request object in the interactive shell.  Furthermore
@@ -463,13 +473,8 @@ class RequestContext(object):
         # See flask.testing for how this works.
         self.auto_pop(exc_value)
 
-        if BROKEN_PYPY_CTXMGR_EXIT and exc_type is not None:
-            reraise(exc_type, exc_value, tb)
-
-    def __repr__(self):
-        return "<%s '%s' [%s] of %s>" % (
-            self.__class__.__name__,
-            self.request.url,
-            self.request.method,
-            self.app.name,
+    def __repr__(self) -> str:
+        return (
+            f"<{type(self).__name__} {self.request.url!r}"
+            f" [{self.request.method}] of {self.app.name}>"
         )
