@@ -1,20 +1,12 @@
-# -*- coding: utf-8 -*-
-"""
-    tests.test_cli
-    ~~~~~~~~~~~~~~
-
-    :copyright: 2010 Pallets
-    :license: BSD-3-Clause
-"""
 # This file was part of Flask-CLI and was modified under the terms of
 # its Revised BSD License. Copyright © 2015 CERN.
-from __future__ import absolute_import
-
 import os
 import ssl
 import sys
 import types
 from functools import partial
+from pathlib import Path
+from unittest.mock import patch
 
 import click
 import pytest
@@ -31,14 +23,15 @@ from flask.cli import FlaskGroup
 from flask.cli import get_version
 from flask.cli import load_dotenv
 from flask.cli import locate_app
+from flask.cli import main as cli_main
 from flask.cli import NoAppException
 from flask.cli import prepare_import
 from flask.cli import run_command
 from flask.cli import ScriptInfo
 from flask.cli import with_appcontext
 
-cwd = os.getcwd()
-test_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "test_apps"))
+cwd = Path.cwd()
+test_path = (Path(__file__) / ".." / "test_apps").resolve()
 
 
 @pytest.fixture
@@ -54,7 +47,6 @@ def test_cli_name(test_apps):
 
 
 def test_find_best_app(test_apps):
-    """Test if `find_best_app` behaves as expected with different combinations of input."""  # noqa: B950
     script_info = ScriptInfo()
 
     class Module:
@@ -77,32 +69,40 @@ def test_find_best_app(test_apps):
         def create_app():
             return Flask("appname")
 
-    assert isinstance(find_best_app(script_info, Module), Flask)
-    assert find_best_app(script_info, Module).name == "appname"
+    app = find_best_app(script_info, Module)
+    assert isinstance(app, Flask)
+    assert app.name == "appname"
 
     class Module:
         @staticmethod
         def create_app(foo):
             return Flask("appname")
 
-    assert isinstance(find_best_app(script_info, Module), Flask)
-    assert find_best_app(script_info, Module).name == "appname"
+    with pytest.deprecated_call(match="Script info"):
+        app = find_best_app(script_info, Module)
+
+    assert isinstance(app, Flask)
+    assert app.name == "appname"
 
     class Module:
         @staticmethod
         def create_app(foo=None, script_info=None):
             return Flask("appname")
 
-    assert isinstance(find_best_app(script_info, Module), Flask)
-    assert find_best_app(script_info, Module).name == "appname"
+    with pytest.deprecated_call(match="script_info"):
+        app = find_best_app(script_info, Module)
+
+    assert isinstance(app, Flask)
+    assert app.name == "appname"
 
     class Module:
         @staticmethod
         def make_app():
             return Flask("appname")
 
-    assert isinstance(find_best_app(script_info, Module), Flask)
-    assert find_best_app(script_info, Module).name == "appname"
+    app = find_best_app(script_info, Module)
+    assert isinstance(app, Flask)
+    assert app.name == "appname"
 
     class Module:
         myapp = Flask("appname1")
@@ -153,29 +153,25 @@ def test_find_best_app(test_apps):
     (
         ("test", cwd, "test"),
         ("test.py", cwd, "test"),
-        ("a/test", os.path.join(cwd, "a"), "test"),
+        ("a/test", cwd / "a", "test"),
         ("test/__init__.py", cwd, "test"),
         ("test/__init__", cwd, "test"),
         # nested package
         (
-            os.path.join(test_path, "cliapp", "inner1", "__init__"),
+            test_path / "cliapp" / "inner1" / "__init__",
             test_path,
             "cliapp.inner1",
         ),
         (
-            os.path.join(test_path, "cliapp", "inner1", "inner2"),
+            test_path / "cliapp" / "inner1" / "inner2",
             test_path,
             "cliapp.inner1.inner2",
         ),
         # dotted name
         ("test.a.b", cwd, "test.a.b"),
-        (os.path.join(test_path, "cliapp.app"), test_path, "cliapp.app"),
+        (test_path / "cliapp.app", test_path, "cliapp.app"),
         # not a Python file, will be caught during import
-        (
-            os.path.join(test_path, "cliapp", "message.txt"),
-            test_path,
-            "cliapp.message.txt",
-        ),
+        (test_path / "cliapp" / "message.txt", test_path, "cliapp.message.txt"),
     ),
 )
 def test_prepare_import(request, value, path, result):
@@ -194,7 +190,7 @@ def test_prepare_import(request, value, path, result):
     request.addfinalizer(reset_path)
 
     assert prepare_import(value) == result
-    assert sys.path[0] == path
+    assert sys.path[0] == str(path)
 
 
 @pytest.mark.parametrize(
@@ -205,19 +201,15 @@ def test_prepare_import(request, value, path, result):
         ("cliapp.factory", None, "app"),
         ("cliapp.factory", "create_app", "app"),
         ("cliapp.factory", "create_app()", "app"),
-        # no script_info
         ("cliapp.factory", 'create_app2("foo", "bar")', "app2_foo_bar"),
         # trailing comma space
         ("cliapp.factory", 'create_app2("foo", "bar", )', "app2_foo_bar"),
-        # takes script_info
-        ("cliapp.factory", 'create_app3("foo")', "app3_foo_spam"),
         # strip whitespace
         ("cliapp.factory", " create_app () ", "app"),
     ),
 )
 def test_locate_app(test_apps, iname, aname, result):
     info = ScriptInfo()
-    info.data["test"] = "spam"
     assert locate_app(info, iname, aname).name == result
 
 
@@ -246,7 +238,7 @@ def test_locate_app_raises(test_apps, iname, aname):
         locate_app(info, iname, aname)
 
 
-def test_locate_app_suppress_raise():
+def test_locate_app_suppress_raise(test_apps):
     info = ScriptInfo()
     app = locate_app(info, "notanapp.py", None, raise_if_not_found=False)
     assert app is None
@@ -261,7 +253,7 @@ def test_get_version(test_apps, capsys):
     from werkzeug import __version__ as werkzeug_version
     from platform import python_version
 
-    class MockCtx(object):
+    class MockCtx:
         resilient_parsing = False
         color = None
 
@@ -271,32 +263,30 @@ def test_get_version(test_apps, capsys):
     ctx = MockCtx()
     get_version(ctx, None, "test")
     out, err = capsys.readouterr()
-    assert "Python " + python_version() in out
-    assert "Flask " + flask_version in out
-    assert "Werkzeug " + werkzeug_version in out
+    assert f"Python {python_version()}" in out
+    assert f"Flask {flask_version}" in out
+    assert f"Werkzeug {werkzeug_version}" in out
 
 
 def test_scriptinfo(test_apps, monkeypatch):
-    """Test of ScriptInfo."""
     obj = ScriptInfo(app_import_path="cliapp.app:testapp")
     app = obj.load_app()
     assert app.name == "testapp"
     assert obj.load_app() is app
 
     # import app with module's absolute path
-    cli_app_path = os.path.abspath(
-        os.path.join(os.path.dirname(__file__), "test_apps", "cliapp", "app.py")
-    )
+    cli_app_path = str(test_path / "cliapp" / "app.py")
+
     obj = ScriptInfo(app_import_path=cli_app_path)
     app = obj.load_app()
     assert app.name == "testapp"
     assert obj.load_app() is app
-    obj = ScriptInfo(app_import_path=cli_app_path + ":testapp")
+    obj = ScriptInfo(app_import_path=f"{cli_app_path}:testapp")
     app = obj.load_app()
     assert app.name == "testapp"
     assert obj.load_app() is app
 
-    def create_app(info):
+    def create_app():
         return Flask("createapp")
 
     obj = ScriptInfo(create_app=create_app)
@@ -308,33 +298,25 @@ def test_scriptinfo(test_apps, monkeypatch):
     pytest.raises(NoAppException, obj.load_app)
 
     # import app from wsgi.py in current directory
-    monkeypatch.chdir(
-        os.path.abspath(
-            os.path.join(os.path.dirname(__file__), "test_apps", "helloworld")
-        )
-    )
+    monkeypatch.chdir(test_path / "helloworld")
     obj = ScriptInfo()
     app = obj.load_app()
     assert app.name == "hello"
 
     # import app from app.py in current directory
-    monkeypatch.chdir(
-        os.path.abspath(os.path.join(os.path.dirname(__file__), "test_apps", "cliapp"))
-    )
+    monkeypatch.chdir(test_path / "cliapp")
     obj = ScriptInfo()
     app = obj.load_app()
     assert app.name == "testapp"
 
 
 def test_with_appcontext(runner):
-    """Test of with_appcontext."""
-
     @click.command()
     @with_appcontext
     def testcmd():
         click.echo(current_app.name)
 
-    obj = ScriptInfo(create_app=lambda info: Flask("testapp"))
+    obj = ScriptInfo(create_app=lambda: Flask("testapp"))
 
     result = runner.invoke(testcmd, obj=obj)
     assert result.exit_code == 0
@@ -342,8 +324,6 @@ def test_with_appcontext(runner):
 
 
 def test_appgroup(runner):
-    """Test of with_appcontext."""
-
     @click.group(cls=AppGroup)
     def cli():
         pass
@@ -360,7 +340,7 @@ def test_appgroup(runner):
     def test2():
         click.echo(current_app.name)
 
-    obj = ScriptInfo(create_app=lambda info: Flask("testappgroup"))
+    obj = ScriptInfo(create_app=lambda: Flask("testappgroup"))
 
     result = runner.invoke(cli, ["test"], obj=obj)
     assert result.exit_code == 0
@@ -372,9 +352,7 @@ def test_appgroup(runner):
 
 
 def test_flaskgroup(runner):
-    """Test FlaskGroup."""
-
-    def create_app(info):
+    def create_app():
         return Flask("flaskgroup")
 
     @click.group(cls=FlaskGroup, create_app=create_app)
@@ -392,9 +370,7 @@ def test_flaskgroup(runner):
 
 @pytest.mark.parametrize("set_debug_flag", (True, False))
 def test_flaskgroup_debug(runner, set_debug_flag):
-    """Test FlaskGroup debug flag behavior."""
-
-    def create_app(info):
+    def create_app():
         app = Flask("flaskgroup")
         app.debug = True
         return app
@@ -409,30 +385,45 @@ def test_flaskgroup_debug(runner, set_debug_flag):
 
     result = runner.invoke(cli, ["test"])
     assert result.exit_code == 0
-    assert result.output == "%s\n" % str(not set_debug_flag)
+    assert result.output == f"{not set_debug_flag}\n"
 
 
-def test_print_exceptions(runner):
-    """Print the stacktrace if the CLI."""
+def test_no_command_echo_loading_error():
+    from flask.cli import cli
 
-    def create_app(info):
-        raise Exception("oh no")
-        return Flask("flaskgroup")
+    runner = CliRunner(mix_stderr=False)
+    result = runner.invoke(cli, ["missing"])
+    assert result.exit_code == 2
+    assert "FLASK_APP" in result.stderr
+    assert "Usage:" in result.stderr
 
-    @click.group(cls=FlaskGroup, create_app=create_app)
-    def cli(**params):
-        pass
 
+def test_help_echo_loading_error():
+    from flask.cli import cli
+
+    runner = CliRunner(mix_stderr=False)
     result = runner.invoke(cli, ["--help"])
     assert result.exit_code == 0
-    assert "Exception: oh no" in result.output
-    assert "Traceback" in result.output
+    assert "FLASK_APP" in result.stderr
+    assert "Usage:" in result.stdout
+
+
+def test_help_echo_exception():
+    def create_app():
+        raise Exception("oh no")
+
+    cli = FlaskGroup(create_app=create_app)
+    runner = CliRunner(mix_stderr=False)
+    result = runner.invoke(cli, ["--help"])
+    assert result.exit_code == 0
+    assert "Exception: oh no" in result.stderr
+    assert "Usage:" in result.stdout
 
 
 class TestRoutes:
     @pytest.fixture
     def invoke(self, runner):
-        def create_app(info):
+        def create_app():
             app = Flask(__name__)
             app.testing = True
 
@@ -451,7 +442,7 @@ class TestRoutes:
 
     @pytest.fixture
     def invoke_no_routes(self, runner):
-        def create_app(info):
+        def create_app():
             app = Flask(__name__, static_folder=None)
             app.testing = True
 
@@ -506,13 +497,13 @@ need_dotenv = pytest.mark.skipif(dotenv is None, reason="dotenv is not installed
 @need_dotenv
 def test_load_dotenv(monkeypatch):
     # can't use monkeypatch.delitem since the keys don't exist yet
-    for item in ("FOO", "BAR", "SPAM"):
+    for item in ("FOO", "BAR", "SPAM", "HAM"):
         monkeypatch._setitem.append((os.environ, item, notset))
 
     monkeypatch.setenv("EGGS", "3")
-    monkeypatch.chdir(os.path.join(test_path, "cliapp", "inner1"))
+    monkeypatch.chdir(test_path)
     assert load_dotenv()
-    assert os.getcwd() == test_path
+    assert Path.cwd() == test_path
     # .flaskenv doesn't overwrite .env
     assert os.environ["FOO"] == "env"
     # set only in .flaskenv
@@ -521,7 +512,8 @@ def test_load_dotenv(monkeypatch):
     assert os.environ["SPAM"] == "1"
     # set manually, files don't overwrite
     assert os.environ["EGGS"] == "3"
-
+    # test env file encoding
+    assert os.environ["HAM"] == "火腿"
     # Non existent file should not load
     assert not load_dotenv("non-existent-file")
 
@@ -531,9 +523,8 @@ def test_dotenv_path(monkeypatch):
     for item in ("FOO", "BAR", "EGGS"):
         monkeypatch._setitem.append((os.environ, item, notset))
 
-    cwd = os.getcwd()
-    load_dotenv(os.path.join(test_path, ".flaskenv"))
-    assert os.getcwd() == cwd
+    load_dotenv(test_path / ".flaskenv")
+    assert Path.cwd() == cwd
     assert "FOO" in os.environ
 
 
@@ -566,14 +557,14 @@ def test_run_cert_path():
 
 
 def test_run_cert_adhoc(monkeypatch):
-    monkeypatch.setitem(sys.modules, "OpenSSL", None)
+    monkeypatch.setitem(sys.modules, "cryptography", None)
 
-    # pyOpenSSL not installed
+    # cryptography not installed
     with pytest.raises(click.BadParameter):
         run_command.make_context("run", ["--cert", "adhoc"])
 
-    # pyOpenSSL installed
-    monkeypatch.setitem(sys.modules, "OpenSSL", types.ModuleType("OpenSSL"))
+    # cryptography installed
+    monkeypatch.setitem(sys.modules, "cryptography", types.ModuleType("cryptography"))
     ctx = run_command.make_context("run", ["--cert", "adhoc"])
     assert ctx.params["cert"] == "adhoc"
 
@@ -589,16 +580,11 @@ def test_run_cert_import(monkeypatch):
     with pytest.raises(click.BadParameter):
         run_command.make_context("run", ["--cert", "not_here"])
 
-    # not an SSLContext
-    if sys.version_info >= (2, 7, 9):
-        with pytest.raises(click.BadParameter):
-            run_command.make_context("run", ["--cert", "flask"])
+    with pytest.raises(click.BadParameter):
+        run_command.make_context("run", ["--cert", "flask"])
 
     # SSLContext
-    if sys.version_info < (2, 7, 9):
-        ssl_context = object()
-    else:
-        ssl_context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
+    ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
 
     monkeypatch.setitem(sys.modules, "ssl_context", ssl_context)
     ctx = run_command.make_context("run", ["--cert", "ssl_context"])
@@ -664,4 +650,12 @@ def test_cli_empty(app):
     app.register_blueprint(bp)
 
     result = app.test_cli_runner().invoke(args=["blue", "--help"])
-    assert result.exit_code == 2, "Unexpected success:\n\n" + result.output
+    assert result.exit_code == 2, f"Unexpected success:\n\n{result.output}"
+
+
+def test_click_7_deprecated():
+    with patch("flask.cli.cli"):
+        if int(click.__version__[0]) < 8:
+            pytest.deprecated_call(cli_main, match=".* Click 7 is deprecated")
+        else:
+            cli_main()
