@@ -1,11 +1,4 @@
-# -*- coding: utf-8 -*-
-"""
-    tests.test_config
-    ~~~~~~~~~~~~~~~~~
-
-    :copyright: 2010 Pallets
-    :license: BSD-3-Clause
-"""
+import json
 import os
 import textwrap
 from datetime import timedelta
@@ -13,7 +6,6 @@ from datetime import timedelta
 import pytest
 
 import flask
-from flask._compat import PY2
 
 
 # config keys used for the TestConfig
@@ -27,9 +19,9 @@ def common_object_test(app):
     assert "TestConfig" not in app.config
 
 
-def test_config_from_file():
+def test_config_from_pyfile():
     app = flask.Flask(__name__)
-    app.config.from_pyfile(__file__.rsplit(".", 1)[0] + ".py")
+    app.config.from_pyfile(f"{__file__.rsplit('.', 1)[0]}.py")
     common_object_test(app)
 
 
@@ -39,10 +31,10 @@ def test_config_from_object():
     common_object_test(app)
 
 
-def test_config_from_json():
+def test_config_from_file():
     app = flask.Flask(__name__)
     current_dir = os.path.dirname(os.path.abspath(__file__))
-    app.config.from_json(os.path.join(current_dir, "static", "config.json"))
+    app.config.from_file(os.path.join(current_dir, "static", "config.json"), json.load)
     common_object_test(app)
 
 
@@ -65,7 +57,7 @@ def test_config_from_mapping():
 
 
 def test_config_from_class():
-    class Base(object):
+    class Base:
         TEST_KEY = "foo"
 
     class Test(Base):
@@ -85,7 +77,7 @@ def test_config_from_envvar(monkeypatch):
     assert not app.config.from_envvar("FOO_SETTINGS", silent=True)
 
     monkeypatch.setattr(
-        "os.environ", {"FOO_SETTINGS": __file__.rsplit(".", 1)[0] + ".py"}
+        "os.environ", {"FOO_SETTINGS": f"{__file__.rsplit('.', 1)[0]}.py"}
     )
     assert app.config.from_envvar("FOO_SETTINGS")
     common_object_test(app)
@@ -116,16 +108,16 @@ def test_config_missing():
     assert not app.config.from_pyfile("missing.cfg", silent=True)
 
 
-def test_config_missing_json():
+def test_config_missing_file():
     app = flask.Flask(__name__)
     with pytest.raises(IOError) as e:
-        app.config.from_json("missing.json")
+        app.config.from_file("missing.json", load=json.load)
     msg = str(e.value)
     assert msg.startswith(
         "[Errno 2] Unable to load configuration file (No such file or directory):"
     )
     assert msg.endswith("missing.json'")
-    assert not app.config.from_json("missing.json", silent=True)
+    assert not app.config.from_file("missing.json", load=json.load, silent=True)
 
 
 def test_custom_config_class():
@@ -186,17 +178,13 @@ def test_from_pyfile_weird_encoding(tmpdir, encoding):
     f = tmpdir.join("my_config.py")
     f.write_binary(
         textwrap.dedent(
-            u"""
-    # -*- coding: {0} -*-
-    TEST_VALUE = "föö"
-    """.format(
-                encoding
-            )
+            f"""
+            # -*- coding: {encoding} -*-
+            TEST_VALUE = "föö"
+            """
         ).encode(encoding)
     )
     app = flask.Flask(__name__)
     app.config.from_pyfile(str(f))
     value = app.config["TEST_VALUE"]
-    if PY2:
-        value = value.decode(encoding)
-    assert value == u"föö"
+    assert value == "föö"
