@@ -1,13 +1,3 @@
-# -*- coding: utf-8 -*-
-"""
-    tests.signals
-    ~~~~~~~~~~~~~~~~~~~~~~~
-
-    Signalling.
-
-    :copyright: 2010 Pallets
-    :license: BSD-3-Clause
-"""
 import pytest
 
 try:
